@@ -35,7 +35,7 @@ class DatabaseInspector(metaclass=ABCMeta):
                                 
     def get_indices(self):
         '''Return names of all indices in the database.'''
-        return dict((row[0], Index(row[0])) 
+        return dict(self.prepare_index(row)
                     for row in self._select(self._INDEX_NAMES_SQL))
         
     def get_procedures(self): 
@@ -51,7 +51,7 @@ class DatabaseInspector(metaclass=ABCMeta):
                     for row in self._select(self._PROCEDURE_NAMES_SQL))
 
     def get_index_columns(self, index):
-        sql = self._INDEX_COLUMNS_SQL % index.name
+        sql = self._INDEX_COLUMNS_SQL % index.base_name
         return tuple(row[0] for row in self._select(sql))        
 
     def build_columns(self, schema_object):
@@ -62,11 +62,6 @@ class DatabaseInspector(metaclass=ABCMeta):
             name = (row[0] if case_sensitve else row[0].lower())
             columns[name] = self.prepare_column(row)
         schema_object.columns = columns
-
-    def build_indices(self, table):
-        sql = self._TABLE_INDEX_NAMES_SQL % table.name
-        table.indices = dict((row[0], Index(row[0], inspector=self)) 
-                             for row in self._select(sql))
                              
     def prepare_default(self, data_type, value):
         if data_type in self.INTEGER_TYPES:
@@ -88,13 +83,23 @@ class DatabaseInspector(metaclass=ABCMeta):
         return True
                     
     def _select(self, sql):
-        connection = self._api.connect(*self._args, **self._kwargs)
-        cursor = connection.cursor()
-        cursor.execute(sql)
+        try:
+            connection = self._api.connect(*self._args, **self._kwargs)
+            cursor = connection.cursor()
+            cursor.execute(sql)
+        except Exception as e: # TODO: properly catch exceptions here
+            raise FathomError(str(e))
         rows = list(cursor)
         connection.close()
         return rows
         
+    def drop_table(self, table):
+        connection = self._api.connect(*self._args, **self._kwargs)
+        cursor = connection.cursor()
+        cursor.execute('DROP TABLE %s' % table.name)
+        connection.commit()
+        connection.close()
+        
     def case(self, string):
         if self.CASE_SENSITIVITY != constants.CASE_INSENSITIVE:
             return string
@@ -119,12 +124,24 @@ class SqliteInspector(DatabaseInspector):
 SELECT name
 FROM sqlite_master
 WHERE type = 'trigger'"""
+
+    _INDEX_NAMES_SQL = """
+SELECT name, tbl_name
+FROM sqlite_master
+WHERE type = 'index'
+"""
+
+    _INDEX_UNIQUENESS_SQL = """
+pragma index_list(%s)
+"""
     
-    _COLUMN_NAMES_SQL = """pragma table_info(%s)"""
-    
-    _TABLE_INDEX_NAMES_SQL = """pragma index_list(%s)"""
-    
-    _INDEX_COLUMNS_SQL = """pragma index_info(%s)"""
+    _COLUMN_NAMES_SQL = """
+pragma table_info(%s)
+"""
+
+    _INDEX_COLUMNS_SQL = """
+pragma index_info(%s)
+"""
     
     _FOREIGN_KEYS_SQL = """pragma foreign_key_list(%s)"""
     
@@ -148,17 +165,20 @@ WHERE type='trigger' AND name = '%s'"""
 
     def get_procedures(self):
         return {}
+
+    def prepare_index(self, row):
+        index = Index(row[0], row[1], inspector=self)
+        for index_row in self._select(self._INDEX_UNIQUENESS_SQL % row[1]):
+            if index_row[1] == row[0]:
+                index.is_unique = (index_row[2] == 1)
+                break
+        return row[0], index    
         
     def build_columns(self, schema_object):
         sql = self._COLUMN_NAMES_SQL % schema_object.name
         schema_object.columns = dict((row[1].lower(), self.prepare_column(row)) 
                                      for row in self._select(sql))
-                                     
-    def build_indices(self, table):
-        sql = self._TABLE_INDEX_NAMES_SQL % table.name
-        table.indices = dict((row[1], Index(row[1], inspector=self))
-                             for row in self._select(sql))
-                             
+
     def build_trigger(self, trigger):
         sql = self._TRIGGER_SQL % trigger.name
         source_sql = self._select(sql)[0][0]
@@ -240,14 +260,15 @@ FROM information_schema.columns
 WHERE table_name = '%s'"""
                            
     _INDEX_NAMES_SQL = """
-SELECT indexname
-FROM pg_indexes
-WHERE schemaname = 'public'"""
-
-    _TABLE_INDEX_NAMES_SQL = """
-SELECT indexname 
-FROM pg_indexes 
-WHERE schemaname='public' AND tablename='%s'
+SELECT i.relname AS indexname, c.relname AS tablename, x.indisunique,  
+       pg_get_indexdef(i.oid) AS indexdef
+FROM pg_index x
+JOIN pg_class c ON c.oid = x.indrelid
+JOIN pg_class i ON i.oid = x.indexrelid
+LEFT JOIN pg_namespace n ON n.oid = c.relnamespace
+LEFT JOIN pg_tablespace t ON t.oid = i.reltablespace
+WHERE c.relkind = 'r'::"char" AND i.relkind = 'i'::"char" AND 
+      n.nspname = 'public';
 """
 
     _PROCEDURE_NAMES_SQL = """
@@ -257,7 +278,7 @@ WHERE pg_language.lanname = 'plpgsql'
 """
 
     _PROCEDURE_ARGUMENTS_SQL = """
-SELECT proargnames, proargtypes
+SELECT proargnames, proargtypes, proargmodes
 FROM pg_proc JOIN pg_language ON pg_proc.prolang = pg_language.oid
 WHERE pg_language.lanname = 'plpgsql' AND proname = '%s' AND proargtypes='%s'
 """
@@ -286,6 +307,10 @@ SELECT column_name
 FROM information_schema.columns
 WHERE table_name = '%s' AND ordinal_position IN ('%s')"""
 
+    _VERSION_SQL = """
+SELECT version()
+"""
+
     _BEFORE_BIT = 2
     _INSERT_BIT, _DELETE_BIT, _UPDATE_BIT = 4, 8, 16
     
@@ -293,6 +318,16 @@ WHERE table_name = '%s' AND ordinal_position IN ('%s')"""
         DatabaseInspector.__init__(self, *db_params)
         import psycopg2
         self._api = psycopg2
+        self.set_version()
+
+    def set_version(self):
+        try:
+            version = self._select(self._VERSION_SQL)[0][0]
+            version = version[len('PostgreSQL') + 1:version.index('on')]
+            self.version = tuple([int(step) for step in version.split('.')])
+        except Exception as e:
+            print('Warning: failed to obtain MySQL version; assuming 8.4.0')
+            version = (8, 4, 0)
                              
     def build_procedure(self, procedure):
         arg_type_oids = procedure._private['arg_type_oids']
@@ -378,6 +413,12 @@ WHERE table_name = '%s' AND ordinal_position IN ('%s')"""
         procedure._private['arg_type_oids'] = row[1]
         return name, procedure
         
+    def prepare_index(self, row):
+        name = '%s: %s' % (row[1], row[0])
+        index = Index(name, row[1], base_name=row[0], inspector=self)
+        index.is_unique = row[2]
+        return name, index        
+        
     def types_from_oids(self, oids):
         return [self._select(self._TYPE_SQL % oid)[0][0] for oid in oids]
     
@@ -407,6 +448,12 @@ FROM information_schema.views"""
     _PROCEDURE_NAMES_SQL = """
 SELECT routine_name, dtd_identifier, routine_definition
 FROM information_schema.routines
+"""
+
+    _PROCEDURE_ARGUMENTS_SQL = """
+SELECT parameter_name, data_type, character_maximum_length, parameter_mode
+FROM information_schema.parameters
+WHERE specific_name = '%s' AND parameter_name IS NOT NULL
 """
 
     _TRIGGER_NAMES_SQL = """
@@ -421,10 +468,9 @@ FROM information_schema.columns
 WHERE table_name = '%s'
 """
 
-    _TABLE_INDEX_NAMES_SQL = """
-SELECT index_name 
+    _INDEX_NAMES_SQL = """
+SELECT index_name, table_name, non_unique
 FROM information_schema.statistics
-WHERE table_name = '%s'
 """
 
     _INDEX_COLUMNS_SQL = """
@@ -478,6 +524,12 @@ WHERE table_name = '%s'
         procedure._private['arg_type_oids'] = row[1]
         return row[0], procedure
         
+    def prepare_index(self, row):
+        name = '%s: %s' % (row[1], row[0])
+        index = Index(name, row[1], base_name=row[0], inspector=self)
+        index.is_unique = not row[2]
+        return name, index
+        
     def get_triggers(self):
         '''Returns names of all triggers in the database.'''
         triggers = {}
@@ -490,10 +542,12 @@ WHERE table_name = '%s'
         return triggers
         
     def build_procedure(self, procedure):
+        procedure.arguments = {}
         if self.supports_routine_parametres():
-            procedure.arguments = {} # i need mysql 5.5 for this
-        else:
-            procedure.arguments = {}
+            # needs mysql 5.5 for this
+            sql = self._PROCEDURE_ARGUMENTS_SQL % procedure.name
+            for row in self._select(sql):
+                procedure.arguments[row[0]] = Argument(row[0], row[1])
 
     def build_foreign_keys(self, table):
         sql = self._FOREIGN_KEYS_SQL % table.name
@@ -568,11 +622,16 @@ SELECT column_name, data_type, data_length, data_default,
 FROM all_tab_columns
 WHERE table_name = '%s'
 """
-    
-    _TABLE_INDEX_NAMES_SQL = """
-SELECT index_name
+   
+    _INDEX_NAMES_SQL = """
+SELECT index_name, table_name, uniqueness
 FROM user_indexes
-WHERE table_name = upper('%s')
+"""
+
+    _INDEX_COLUMNS_SQL = """
+SELECT column_name
+FROM user_ind_columns
+WHERE index_name = '%s'
 """
 
     _TRIGGER_INFO_SQL = """
@@ -596,8 +655,8 @@ WHERE constraint_name = upper('%s')
 ORDER BY position
 """
 
-    def __init__(self, *db_params):
-        DatabaseInspector.__init__(self, *db_params)
+    def __init__(self, *args, **kwargs):
+        DatabaseInspector.__init__(self, *args, **kwargs)
         import cx_Oracle
         self._api = cx_Oracle
         
@@ -627,6 +686,12 @@ ORDER BY position
         sql = self._ARGUMENTS_SQL % procedure.name
         procedure.arguments = {row[0]: Argument(row[0], row[1]) 
                                for row in self._select(sql)}
+                               
+    def prepare_index(self, row):
+        name = '%s: %s' % (row[1], row[0])
+        index = Index(name, row[1], base_name=row[0], inspector=self)
+        index.is_unique = (row[2] == 'UNIQUE')
+        return name, index                               
                 
     def build_foreign_keys(self, table):
         table.foreign_keys = []
